@@ -351,8 +351,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
         }
     }
 
-    unsafe fn fill_buffer(&mut self, buffer: &super::Buffer, range: crate::MemoryRange, value: u8) {
-        assert_eq!(value, 0, "Only zero is supported!");
+    unsafe fn clear_buffer(&mut self, buffer: &super::Buffer, range: crate::MemoryRange) {
         let list = self.list.unwrap();
         let mut offset = range.start;
         while offset < range.end {
@@ -368,6 +367,100 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
         }
     }
 
+    unsafe fn clear_texture(
+        &mut self,
+        texture: &super::Texture,
+        subresource_range: &wgt::ImageSubresourceRange,
+    ) {
+        // Note that CopyTextureRegion for depth/stencil or multisample resources would require full subresource copies.
+        // Meaning we'd need a much larger pre-zeroed buffer
+        // (but instead we just define clear_texture to not support these)
+
+        let list = self.list.unwrap();
+        let mut src_location = d3d12::D3D12_TEXTURE_COPY_LOCATION {
+            pResource: self.shared.zero_buffer.as_mut_ptr(),
+            Type: d3d12::D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT,
+            u: mem::zeroed(),
+        };
+        let mut dst_location = d3d12::D3D12_TEXTURE_COPY_LOCATION {
+            pResource: texture.resource.as_mut_ptr(),
+            Type: d3d12::D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX,
+            u: mem::zeroed(),
+        };
+        let raw_format = conv::map_texture_format(texture.format);
+        let format_desc = texture.format.describe();
+
+        let mip_range = subresource_range.base_mip_level..match subresource_range.mip_level_count {
+            Some(c) => subresource_range.base_mip_level + c.get(),
+            None => texture.mip_level_count,
+        };
+        let array_range = subresource_range.base_array_layer
+            ..match subresource_range.array_layer_count {
+                Some(c) => subresource_range.base_array_layer + c.get(),
+                None => texture.array_layer_count(),
+            };
+        for mip_level in mip_range {
+            let mip_size = texture
+                .size
+                .mip_level_size(mip_level, texture.dimension == wgt::TextureDimension::D3);
+            let depth = if texture.dimension == wgt::TextureDimension::D3 {
+                mip_size.depth_or_array_layers
+            } else {
+                1
+            };
+            let bytes_per_row = mip_size.width / format_desc.block_dimensions.0 as u32
+                * format_desc.block_size as u32;
+            // round up to a multiple of d3d12::D3D12_TEXTURE_DATA_PITCH_ALIGNMENT
+            let bytes_per_row = (bytes_per_row + d3d12::D3D12_TEXTURE_DATA_PITCH_ALIGNMENT - 1)
+                / d3d12::D3D12_TEXTURE_DATA_PITCH_ALIGNMENT
+                * d3d12::D3D12_TEXTURE_DATA_PITCH_ALIGNMENT;
+
+            let max_rows_per_copy = super::ZERO_BUFFER_SIZE as u32 / bytes_per_row;
+            // round down to a multiple of rows needed by the texture format
+            let max_rows_per_copy = max_rows_per_copy / format_desc.block_dimensions.1 as u32
+                * format_desc.block_dimensions.1 as u32;
+            assert!(max_rows_per_copy > 0, "Zero buffer size is too small to fill a single row of a texture with dimension {:?}, size {:?} and format {:?}", texture.dimension, texture.size, texture.format);
+
+            for array_layer in array_range.clone() {
+                // We excluded depth/stencil, so plane should be always zero
+                *dst_location.u.SubresourceIndex_mut() =
+                    texture.calc_subresource(mip_level, array_layer, 0);
+                // 3D textures are quickly massive in memory size, so we don't bother trying to do more than one layer at once.
+                for z in 0..depth {
+                    // May need multiple copies for each subresource!
+                    // We assume that we never need to split a row. Back of the envelope calculation tells us a 512kb byte buffer is enough for this for most extreme known cases.
+                    // max_texture_width * max_pixel_size = 32768 * 16 = 512kb
+                    let mut num_rows_left = mip_size.height;
+                    while num_rows_left > 0 {
+                        let num_rows = num_rows_left.min(max_rows_per_copy);
+
+                        *src_location.u.PlacedFootprint_mut() =
+                            d3d12::D3D12_PLACED_SUBRESOURCE_FOOTPRINT {
+                                Offset: 0,
+                                Footprint: d3d12::D3D12_SUBRESOURCE_FOOTPRINT {
+                                    Format: raw_format,
+                                    Width: mip_size.width,
+                                    Height: num_rows,
+                                    Depth: 1,
+                                    RowPitch: bytes_per_row,
+                                },
+                            };
+
+                        list.CopyTextureRegion(
+                            &dst_location,
+                            0,
+                            mip_size.height - num_rows_left,
+                            z,
+                            &src_location,
+                            std::ptr::null(),
+                        );
+                        num_rows_left -= num_rows;
+                    }
+                }
+            }
+        }
+    }
+
     unsafe fn copy_buffer_to_buffer<T>(
         &mut self,
         src: &super::Buffer,
@@ -674,9 +767,14 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
                 };
                 self.temp.barriers.push(barrier);
             }
-            list.ResourceBarrier(self.temp.barriers.len() as u32, self.temp.barriers.as_ptr());
+
+            if !self.temp.barriers.is_empty() {
+                profiling::scope!("ID3D12GraphicsCommandList::ResourceBarrier");
+                list.ResourceBarrier(self.temp.barriers.len() as u32, self.temp.barriers.as_ptr());
+            }
 
             for resolve in self.pass.resolves.iter() {
+                profiling::scope!("ID3D12GraphicsCommandList::ResolveSubresource");
                 list.ResolveSubresource(
                     resolve.dst.0.as_mut_ptr(),
                     resolve.dst.1,
@@ -691,7 +789,10 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
                 let transition = barrier.u.Transition_mut();
                 mem::swap(&mut transition.StateBefore, &mut transition.StateAfter);
             }
-            list.ResourceBarrier(self.temp.barriers.len() as u32, self.temp.barriers.as_ptr());
+            if !self.temp.barriers.is_empty() {
+                profiling::scope!("ID3D12GraphicsCommandList::ResourceBarrier");
+                list.ResourceBarrier(self.temp.barriers.len() as u32, self.temp.barriers.as_ptr());
+            }
         }
 
         self.end_pass();
