@@ -189,8 +189,17 @@ impl super::Adapter {
         log::debug!("Extensions: {:#?}", extensions);
 
         let ver = Self::parse_version(&version).ok()?;
+        if ver < (3, 0) {
+            log::warn!(
+                "Returned GLES context is {}.{}, when 3.0+ was requested",
+                ver.0,
+                ver.1
+            );
+            return None;
+        }
 
         let supports_storage = ver >= (3, 1);
+
         let shading_language_version = {
             let sl_version = gl.get_parameter_string(glow::SHADING_LANGUAGE_VERSION);
             log::info!("SL version: {}", &sl_version);
@@ -269,6 +278,7 @@ impl super::Adapter {
         downlevel_flags.set(
             wgt::DownlevelFlags::VERTEX_STORAGE,
             max_storage_block_size != 0
+                && max_storage_buffers_per_shader_stage != 0
                 && (vertex_shader_storage_blocks != 0 || vertex_ssbo_false_zero),
         );
         downlevel_flags.set(wgt::DownlevelFlags::FRAGMENT_STORAGE, supports_storage);
@@ -598,7 +608,7 @@ impl crate::Adapter<super::Api> for super::Adapter {
     ) -> Option<crate::SurfaceCapabilities> {
         if surface.presentable {
             Some(crate::SurfaceCapabilities {
-                formats: if surface.enable_srgb {
+                formats: if surface.supports_srgb() {
                     vec![
                         wgt::TextureFormat::Rgba8UnormSrgb,
                         #[cfg(not(target_arch = "wasm32"))]
