@@ -33,6 +33,8 @@
     clippy::vec_init_then_push,
     // "if panic" is a good uniform construct.
     clippy::if_then_panic,
+    // We unsafe impl `Send` for a reason.
+    clippy::non_send_fields_in_send_ty,
     // TODO!
     clippy::missing_safety_doc,
 )]
