@@ -652,9 +652,9 @@ impl crate::Device<super::Api> for super::Device {
     unsafe fn exit(self, queue: super::Queue) {
         self.mem_allocator.into_inner().cleanup(&*self.shared);
         self.desc_allocator.into_inner().cleanup(&*self.shared);
-        self.shared
-            .raw
-            .destroy_semaphore(queue.relay_semaphore, None);
+        for &sem in queue.relay_semaphores.iter() {
+            self.shared.raw.destroy_semaphore(sem, None);
+        }
         self.shared.free_resources();
     }
 
