@@ -1,7 +1,6 @@
 use std::{
-    cmp,
     ffi::{c_void, CStr, CString},
-    mem, slice,
+    slice,
     sync::Arc,
     thread,
 };
@@ -121,7 +120,6 @@ impl super::Swapchain {
 impl super::Instance {
     pub fn required_extensions(
         entry: &ash::Entry,
-        driver_api_version: u32,
         flags: crate::InstanceFlags,
     ) -> Result<Vec<&'static CStr>, crate::InstanceError> {
         let instance_extensions = entry
@@ -161,11 +159,6 @@ impl super::Instance {
 
         extensions.push(vk::KhrGetPhysicalDeviceProperties2Fn::name());
 
-        // VK_KHR_storage_buffer_storage_class required for `Naga` on Vulkan 1.0 devices
-        if driver_api_version == vk::API_VERSION_1_0 {
-            extensions.push(vk::KhrStorageBufferStorageClassFn::name());
-        }
-
         // Only keep available extensions.
         extensions.retain(|&ext| {
             if instance_extensions
@@ -196,14 +189,10 @@ impl super::Instance {
         has_nv_optimus: bool,
         drop_guard: Option<super::DropGuard>,
     ) -> Result<Self, crate::InstanceError> {
-        if driver_api_version == vk::API_VERSION_1_0
-            && !extensions.contains(&vk::KhrStorageBufferStorageClassFn::name())
-        {
-            log::warn!("Required VK_KHR_storage_buffer_storage_class extension is not supported");
-            return Err(crate::InstanceError);
-        }
+        log::info!("Instance version: 0x{:x}", driver_api_version);
 
         let debug_utils = if extensions.contains(&ext::DebugUtils::name()) {
+            log::info!("Enabling debug utils");
             let extension = ext::DebugUtils::new(&entry, &raw_instance);
             let vk_info = vk::DebugUtilsMessengerCreateInfoEXT::builder()
                 .flags(vk::DebugUtilsMessengerCreateFlagsEXT::empty())
@@ -221,16 +210,19 @@ impl super::Instance {
             None
         };
 
-        let get_physical_device_properties = extensions
-            .iter()
-            .find(|&&ext| ext == vk::KhrGetPhysicalDeviceProperties2Fn::name())
-            .map(|_| {
-                vk::KhrGetPhysicalDeviceProperties2Fn::load(|name| {
-                    mem::transmute(
-                        entry.get_instance_proc_addr(raw_instance.handle(), name.as_ptr()),
-                    )
-                })
-            });
+        // We can't use any of Vulkan-1.1+ abilities on Vk 1.0 instance,
+        // so disabling this query helps.
+        let get_physical_device_properties = if driver_api_version >= vk::API_VERSION_1_1
+            && extensions.contains(&khr::GetPhysicalDeviceProperties2::name())
+        {
+            log::info!("Enabling device properties2");
+            Some(khr::GetPhysicalDeviceProperties2::new(
+                &entry,
+                &raw_instance,
+            ))
+        } else {
+            None
+        };
 
         Ok(Self {
             shared: Arc::new(super::InstanceShared {
@@ -458,9 +450,11 @@ impl crate::Instance<super::Api> for super::Instance {
             .application_version(1)
             .engine_name(CStr::from_bytes_with_nul(b"wgpu-hal\0").unwrap())
             .engine_version(2)
-            .api_version({
-                // Pick the latest API version available, but don't go later than the SDK version used by `gfx_backend_vulkan`.
-                cmp::min(driver_api_version, {
+            .api_version(
+                // Vulkan 1.0 doesn't like anything but 1.0 passed in here...
+                if driver_api_version < vk::API_VERSION_1_1 {
+                    vk::API_VERSION_1_0
+                } else {
                     // This is the max Vulkan API version supported by `wgpu-hal`.
                     //
                     // If we want to increment this, there are some things that must be done first:
@@ -470,10 +464,10 @@ impl crate::Instance<super::Api> for super::Instance {
                     //    - If any were obsoleted in the new API version, we must implement a fallback for the new API version
                     //    - If any are non-KHR-vendored, we must ensure the new behavior is still correct (since backwards-compatibility is not guaranteed).
                     vk::HEADER_VERSION_COMPLETE
-                })
-            });
+                },
+            );
 
-        let extensions = Self::required_extensions(&entry, driver_api_version, desc.flags)?;
+        let extensions = Self::required_extensions(&entry, desc.flags)?;
 
         let instance_layers = entry.enumerate_instance_layer_properties().map_err(|e| {
             log::info!("enumerate_instance_layer_properties: {:?}", e);
@@ -606,6 +600,8 @@ impl crate::Instance<super::Api> for super::Instance {
     }
 
     unsafe fn enumerate_adapters(&self) -> Vec<crate::ExposedAdapter<super::Api>> {
+        use crate::auxil::db;
+
         let raw_devices = match self.shared.raw.enumerate_physical_devices() {
             Ok(devices) => devices,
             Err(err) => {
@@ -620,8 +616,11 @@ impl crate::Instance<super::Api> for super::Instance {
             .collect::<Vec<_>>();
 
         // Detect if it's an Intel + NVidia configuration with Optimus
-        if cfg!(target_os = "linux") && self.shared.has_nv_optimus {
-            use crate::auxil::db;
+        let has_nvidia_dgpu = exposed_adapters.iter().any(|exposed| {
+            exposed.info.device_type == wgt::DeviceType::DiscreteGpu
+                && exposed.info.vendor == db::nvidia::VENDOR as usize
+        });
+        if cfg!(target_os = "linux") && has_nvidia_dgpu && self.shared.has_nv_optimus {
             for exposed in exposed_adapters.iter_mut() {
                 if exposed.info.device_type == wgt::DeviceType::IntegratedGpu
                     && exposed.info.vendor == db::intel::VENDOR as usize
