@@ -84,7 +84,7 @@ struct InstanceShared {
     drop_guard: Option<DropGuard>,
     flags: crate::InstanceFlags,
     debug_utils: Option<DebugUtils>,
-    get_physical_device_properties: Option<vk::KhrGetPhysicalDeviceProperties2Fn>,
+    get_physical_device_properties: Option<khr::GetPhysicalDeviceProperties2>,
     entry: ash::Entry,
     has_nv_optimus: bool,
 }
@@ -335,12 +335,13 @@ pub struct Queue {
     swapchain_fn: khr::Swapchain,
     device: Arc<DeviceShared>,
     family_index: u32,
-    /// This special semaphore is used to synchronize GPU work of
-    /// everything on a queue with... itself. Yikes!
-    /// It's required by the confusing portion of the spec to be signalled
-    /// by last submission and waited by the present.
-    relay_semaphore: vk::Semaphore,
-    relay_active: bool,
+    /// We use a redundant chain of semaphores to pass on the signal
+    /// from submissions to the last present, since it's required by the
+    /// specification.
+    /// It would be correct to use a single semaphore there, but
+    /// https://gitlab.freedesktop.org/mesa/mesa/-/issues/5508
+    relay_semaphores: [vk::Semaphore; 2],
+    relay_index: Option<usize>,
 }
 
 #[derive(Debug)]
@@ -565,7 +566,7 @@ impl crate::Queue<Api> for Queue {
 
         let mut fence_raw = vk::Fence::null();
         let mut vk_timeline_info;
-        let mut semaphores = [self.relay_semaphore, vk::Semaphore::null()];
+        let mut signal_semaphores = [vk::Semaphore::null(), vk::Semaphore::null()];
         let signal_values;
 
         if let Some((fence, value)) = signal_fence {
@@ -573,7 +574,7 @@ impl crate::Queue<Api> for Queue {
             match *fence {
                 Fence::TimelineSemaphore(raw) => {
                     signal_values = [!0, value];
-                    semaphores[1] = raw;
+                    signal_semaphores[1] = raw;
                     vk_timeline_info = vk::TimelineSemaphoreSubmitInfo::builder()
                         .signal_semaphore_values(&signal_values);
                     vk_info = vk_info.push_next(&mut vk_timeline_info);
@@ -596,18 +597,24 @@ impl crate::Queue<Api> for Queue {
         }
 
         let wait_stage_mask = [vk::PipelineStageFlags::TOP_OF_PIPE];
-        if self.relay_active {
-            vk_info = vk_info
-                .wait_semaphores(&semaphores[..1])
-                .wait_dst_stage_mask(&wait_stage_mask);
-        }
-        self.relay_active = true;
-        let signal_count = if semaphores[1] == vk::Semaphore::null() {
+        let sem_index = match self.relay_index {
+            Some(old_index) => {
+                vk_info = vk_info
+                    .wait_semaphores(&self.relay_semaphores[old_index..old_index + 1])
+                    .wait_dst_stage_mask(&wait_stage_mask);
+                (old_index + 1) % self.relay_semaphores.len()
+            }
+            None => 0,
+        };
+        self.relay_index = Some(sem_index);
+        signal_semaphores[0] = self.relay_semaphores[sem_index];
+
+        let signal_count = if signal_semaphores[1] == vk::Semaphore::null() {
             1
         } else {
             2
         };
-        vk_info = vk_info.signal_semaphores(&semaphores[..signal_count]);
+        vk_info = vk_info.signal_semaphores(&signal_semaphores[..signal_count]);
 
         profiling::scope!("vkQueueSubmit");
         self.device
@@ -625,14 +632,12 @@ impl crate::Queue<Api> for Queue {
 
         let swapchains = [ssc.raw];
         let image_indices = [texture.index];
-        let semaphores = [self.relay_semaphore];
         let mut vk_info = vk::PresentInfoKHR::builder()
             .swapchains(&swapchains)
             .image_indices(&image_indices);
 
-        if self.relay_active {
-            vk_info = vk_info.wait_semaphores(&semaphores);
-            self.relay_active = false;
+        if let Some(old_index) = self.relay_index.take() {
+            vk_info = vk_info.wait_semaphores(&self.relay_semaphores[old_index..old_index + 1]);
         }
 
         let suboptimal = {
